import json
import responses
from brian_dashboard_manager.grafana import datasource, provision
from brian_dashboard_manager.grafana.utils.request import AdminRequest


@responses.activate
def test_get_datasources(mock_grafana):
    mock_grafana.create_datasource({"some": "data"})
    request = mock_grafana.request

    data = datasource.get_datasources(request)
    assert data[0]["some"] == "data"


@responses.activate
def test_get_missing_datasource_definitions(mock_grafana, tmp_path):
    request = mock_grafana.request

    source = {"some": "data"}
    (tmp_path / "datasource.json").write_text(json.dumps(source))
    assert list(
        datasource.get_missing_datasource_definitions(request, str(tmp_path))
    ) == [source]


def test_datasource_provisioned():
    val = datasource._datasource_exists({}, [])
    assert val

    val = datasource._datasource_exists({'id': 1}, [])
    assert val is False

    val = datasource._datasource_exists({'id': 1, "name": 'testcase2'},
                                        [{'id': -1, 'name': 'testcase1'},
                                         {'id': 1, 'name': 'testcase1'}])
    assert val is False

    val = datasource._datasource_exists({'id': 1},
                                        [{'id': -1, 'name': 'testcase1'},
                                         {'id': 1, 'name': 'testcase2'}])
    assert val

    val = datasource._datasource_exists({'id': 2, "name": 'testcase2'},
                                        [{'id': -1, 'name': 'testcase1'},
                                         {'id': 1, 'name': 'testcase1'},
                                         {'id': 2, 'name': 'testcase2'}])
    assert val


@responses.activate
def test_create_prod_datasource(mock_grafana):
    BODY = {
        "name": "brian-influx-datasource",
        "type": "influxdb",
        "access": "proxy",
        "url": "http://test-brian-datasource.geant.org:8086",
        "database": "test-db",
        "basicAuth": False,
        "isDefault": True,
        "readOnly": False
    }

    request = mock_grafana.request

    assert not mock_grafana.datasources

    result = provision.create_datasource(request, BODY)
    assert result["uid"] in mock_grafana.datasources


@responses.activate
def test_create_prod_datasource_fails(data_config):
    BODY = {
        "name": "brian-influx-datasource",
        "type": "influxdb",
        "access": "proxy",
        "url": "http://test-brian-datasource.geant.org:8086",
        "database": "test-db",
        "basicAuth": False,
        "isDefault": True,
        "readOnly": False
    }

    request = AdminRequest(**data_config)

    responses.add_callback(
        method=responses.POST,
        url=request.BASE_URL + 'api/datasources',
        callback=lambda f: (400, {}, ''))

    data = provision.create_datasource(
        request, BODY)

    # if an error occured when provisioning a datasource, we log the response
    # but return None
    assert data is None
