import responses
from brian_dashboard_manager.grafana import provision


@responses.activate
def test_get_organizations(mock_grafana):
    request = mock_grafana.request

    mock_grafana.create_organization({"id": 91, "name": "Testorg1"})
    mock_grafana.create_organization({"id": 91, "name": "Testorg2"})

    data = provision.get_organizations(request)
    assert [o["name"] for o in data] == ["Testorg1", "Testorg2"]


@responses.activate
def test_create_organization(mock_grafana):
    ORG_NAME = 'fakeorg123'
    request = mock_grafana.request

    assert not len(mock_grafana.organizations)

    data = provision.create_organization(request, ORG_NAME)
    assert len(mock_grafana.organizations) == 1
    assert next(iter(mock_grafana.organizations.values()))["id"] == data["id"]
    assert data["name"] == ORG_NAME


@responses.activate
def test_delete_expired_api_tokens(mock_grafana):
    mock_grafana.create_api_token({"secondsToLive": -1})  # an expired token
    request = mock_grafana.request

    assert len(mock_grafana.api_tokens) == 1
    provision.delete_expired_api_tokens(request)
    assert not mock_grafana.api_tokens


@responses.activate
def test_create_api_token(mock_grafana):
    ORG_ID = 1
    BODY = {
        'name': 'test-token',
        'role': 'Admin',
        'secondsToLive': 3600
    }
    request = mock_grafana.request
    assert not mock_grafana.api_tokens
    provision.create_api_token(request, ORG_ID, BODY)
    assert len(mock_grafana.api_tokens) == 1
    assert next(iter(mock_grafana.api_tokens.values()))["name"] == "test-token"
